<?php

namespace Directus\Db\TableGateway;

use Directus\Acl\Acl;
use Directus\Bootstrap;
use Directus\Db\TableGateway\AclAwareTableGateway;
use Directus\Db\TableGateway\RelationalTableGateway;
use Directus\Db\TableSchema;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Select;
use Zend\Db\Sql\Insert;
use Zend\Db\Sql\Update;

class DirectusBookmarksTableGateway extends AclAwareTableGateway {

    public static $_tableName = "directus_bookmarks";

    public function __construct(Acl $acl, AdapterInterface $adapter) {
        parent::__construct($acl, self::$_tableName, $adapter);
    }

    public static $defaultBookmarksValues = array(
      'Activity' => array(
        "title" => "Activity",
        "url" => "activity",
        "icon_class" => "icon-bell",
        "section" => "other"),
      'Files' => array(
        "title" => "Files",
        "url" => "files",
        "icon_class" => "icon-attach",
        "section" => "other"),
      'Messages' => array(
        "title" => "Messages",
        "url" => "messages",
        "icon_class" => "icon-chat",
        "section" => "other"),
      'Users' => array(
        "title"          => "Users",
        "url"    => "users",
        "icon_class"        => "icon-users",
        "section" => "other")
    );

    public function createDefaultBookmark($title, $bookmark) {
        // Global default values
      if(isset(self::$defaultBookmarksValues[$title])) {
        foreach(self::$defaultBookmarksValues[$title] as $field => $defaultValue) {
          if(!isset($bookmark[$field]) || ("0" !== $bookmark[$field] && empty($bookmark[$field]))) {
            if(!isset($bookmark[$field])) {
                $bookmark[$field] = $defaultValue;
            }
          }
        }
      }
      return $bookmark;
    }

    public function updateBookmark($payload) {
      $update = new Update($this->table);
      $update->set($payload);
      $this->updateWith($update);
    }

    public function insertBookmark($payload) {
      $insert = new Insert($this->table);
      $insert->values($payload);
      $this->insertWith($insert);
      return $this->lastInsertValue;
    }

    public function fetchByUserAndId($user_id, $id) {
        $select = new Select($this->table);
        $select->limit(1);
        $select
            ->where
                ->equalTo('id', $id)
                ->equalTo('user', $user_id);

        $bookmarks = $this
            ->selectWith($select)
            ->current();

        if($bookmarks) {
            $bookmarks = $bookmarks->toArray();
        }

        return $bookmarks;
    }

    public function fetchAllByUser($user_id) {
        $db = Bootstrap::get('olddb');

        $sql =
            'SELECT
                id,
                user,
                title,
                url,
                icon_class,
                section
            FROM
                directus_bookmarks
            WHERE
                directus_bookmarks.user = :user';

        $sth = $db->dbh->prepare($sql);
        $sth->bindValue(':user', $user_id, \PDO::PARAM_INT);
        $sth->execute();

        $bookmarks = array();

        $defaultBookmarks = array_keys(self::$defaultBookmarksValues);

        while ($row = $sth->fetch(\PDO::FETCH_ASSOC)) {
            $title = $row['title'];

            if (($key = array_search($title, $defaultBookmarks)) !== false) unset($defaultBookmarks[$key]);

            if (!isset($row['user'])) {
                $row = null;
            }

            $bookmarks[$title] = $row;
        }
        foreach($defaultBookmarks as $defaultBookmark) {
          $data = array(
            'user' => $user_id
          );

          $row = $this->createDefaultBookmark($defaultBookmark, $data);
          $id = $db->set_entry(self::$_tableName, $row);

          $bookmarks[$defaultBookmark] = $row;
        }

        return $bookmarks;
    }



}
